///////////////////////////////////////////////////////////////
// File:  v810_ds2.cpp
// 
// Author:  Bob VanderClay (vandercl@umich.edu)
//          David Tucker (dbt@dana.ucc.nau.edu)
//
// Description:  Implementation of v810 disassmbler.    
//
// History:
// --------
// 8 Apr 1998 - Created
// 9 Apr 1998 - Version 1.0 completed
// Many Many changes since!
// v1.04 24 Nov 1998


#include "v810_ds2.h"
#include <stdlib.h>
#include <stdio.h>

// Routines to automate the mundane task of sign extending...
WORD sign_26(WORD num) {
	if (num & 0x02000000) return ((WORD)num | 0xFC000000);
	return(num);
}

WORD sign_16(HWORD num) {
	if (num & 0x8000) return ((WORD)num | 0xFFFF0000);
	return(num);
}

WORD sign_9(HWORD num) {
	if (num & 0x0100) return ((WORD)num | 0xFFFFFE00);
	return(num);
}

WORD sign_8(BYTE num) {
	if (num & 0x0080) return ((WORD)num | 0xFFFFFF00);
	return(num);
}

WORD sign_5(BYTE num) {
	if (num & 0x0010) return ((WORD)num | 0xFFFFFFE0);
	return(num);
}


void
main(int argc, char *argv[]) {
	int		lowB, highB, lowB2, highB2;	     // up to 4 bytes for instruction (either 16 or 32 bits)
	int     opcode, arg1, arg2, arg3;
	int     pc = 0;
	int     pc_init = 0;
	int     pc_count = 10000000;

	FILE    *infile, *outfile;

	// Check parameter count.
	if(argc < 3) {
		printf("\nInvalid parameters.\nUSAGE:  v810_dis <inputfile> <outputfile> <Starting Address> <Num Bytes to dissasemble>");
		exit(1);
	}
	if (argc >3) {
	    pc = atoi(argv[3]);
	}

	if (argc >4) {
	    pc_count = atoi(argv[4]);
	}

	pc_init = pc;

	// Open input file.
	if((infile = fopen(argv[1], "rb")) == NULL) {
		printf("\nCannot open input file %s.", argv[1]);
		exit(1);
	}

	// Open output file.
	if((outfile = fopen(argv[2], "w")) == NULL) {
		printf("\nCannot open output file %s.", argv[2]);
		exit(1);
	}

	printf("\nBeginning disassembly.");

	while(!feof(infile)&& pc < pc_count+pc_init) {
		fseek(infile, pc, SEEK_SET);
		lowB = getc(infile);
		fseek(infile, pc + 1, SEEK_SET);
		highB = getc(infile);
		fseek(infile, pc + 2, SEEK_SET);
		lowB2 = getc(infile);
		fseek(infile, pc + 3, SEEK_SET);
		highB2 = getc(infile);

		if(highB == 0xFF) 
		    if(lowB == 0xFF) {
			fprintf(outfile, "\n%08x\t%02x %02x", pc, lowB, highB);
			pc += 2;						
			continue;
		    }
		if(highB == 0x00) 
		    if(lowB== 0x00) {
			fprintf(outfile, "\n%08x\t%02x %02x", pc, lowB, highB);
			pc += 2;						
			continue;
		    }

		opcode = highB >> 2;
		if((highB & 0xE0) == 0x80)	  // Special opcode format for	    
			opcode = (highB >> 1);	      // type III instructions.
		
		if(opcode > 0x4F | opcode < 0) {//Error Invalid opcode!
		    fprintf(outfile, "\n%08x\t%02x %02x", pc, lowB, highB);
		    pc += 2;						
		}
		
		switch(optable[opcode].addr_mode) {
		case AM_I:
			arg1 = (lowB >> 5) + ((highB & 0x3) << 3);
			arg2 = (lowB & 0x1F);
			if (opcode == JMP) {
				fprintf(outfile, "\n%08x\t%s\t\t[$%d]", pc, optable[opcode].opname, arg2);
			} else {
				fprintf(outfile, "\n%08x\t%s\t\t$%d, $%d", pc, optable[opcode].opname, arg2, arg1);
			}
			pc += 2;   // 16 bit instruction
			break;
		case AM_II:
			arg1 = (lowB >> 5) + ((highB & 0x3) << 3);
			arg2 = (lowB & 0x1F);
			fprintf(outfile, "\n%08x\t%s\t\t%d, $%d", pc, optable[opcode].opname, sign_5(arg2), arg1);
			pc += 2;   // 16 bit instruction
			break;
		case AM_III:
			arg1 = ((highB & 0x1) << 8) + (lowB & 0xFE);
			fprintf(outfile, "\n%08x\t%s\t\t%d", pc, optable[opcode].opname, sign_9(arg1));
			pc += 2;   // 16 bit instruction
			break;
		case AM_IV:
			arg1 = ((highB & 0x3) << 24) + (lowB << 16) + (highB2 << 8) + lowB2;
			fprintf(outfile, "\n%08x\t%s\t\t%d", pc, optable[opcode].opname, sign_26(arg1));
			pc += 4;					       // 32 bit instruction
			break;
		case AM_V:       
			arg1 = (lowB >> 5) + ((highB & 0x3) << 3);
			arg2 = (lowB & 0x1F);
			arg3 = (highB2 << 8) + lowB2;
			fprintf(outfile, "\n%08x\t%s\t\t%02x, $%d, $%d", pc, optable[opcode].opname, arg3, arg2, arg1 );
			pc += 4;   // 32 bit instruction
			break;
		case AM_VIa:  // Mode6 form1
			arg1 = (lowB >> 5) + ((highB & 0x3) << 3);
			arg2 = (lowB & 0x1F);
			arg3 = (highB2 << 8) + lowB2;
			fprintf(outfile, "\n%08x\t%s\t\t%d[$%d], $%d", pc, optable[opcode].opname, sign_16(arg3), arg2, arg1);
			pc += 4;   // 32 bit instruction
			break;
		case AM_VIb:  // Mode6 form2
			arg1 = (lowB >> 5) + ((highB & 0x3) << 3);
			arg2 = (lowB & 0x1F);
			arg3 = (highB2 << 8) + lowB2;			      //  whats the order??? 2,3,1 or 1,3,2
			fprintf(outfile, "\n%08x\t%s\t\t$%d, %d[$%d]", pc, optable[opcode].opname, arg1, sign_16(arg3), arg2);
			pc += 4;   // 32 bit instruction
			break;
		case AM_VII:   // Unhandled
			fprintf(outfile, "\n%08x\t %02x %02x %02x %02x", pc, lowB, highB, lowB2, highB2);
			pc +=4;	// 32 bit instruction
			break;
		case AM_VIII:  // Unhandled
			fprintf(outfile, "\n%08x\t %02x %02x %02x %02x", pc, lowB, highB, lowB2, highB2);
			pc += 4;   // 32 bit instruction
			break;
		case AM_IX:
			arg1 = (lowB & 0x1); // Mode ID, Ignore for now
			fprintf(outfile, "\n%08x\t%s", pc, optable[opcode].opname);   
			pc += 2;   // 16 bit instruction
			break;
		case AM_BSTR:  // Bit String Subopcodes
			arg1 = (lowB >> 5) + ((highB & 0x3) << 3);
			arg2 = (lowB & 0x1F);
			if(arg2 > 16) {
				fprintf(outfile, "\n%08x\tError", pc);
			} else {
				fprintf(outfile, "\n%08x\t%s", pc, bssuboptable[arg2]);
			}
			pc += 2;   // 16 bit instruction
			break;
		case AM_FPP:   // Floating Point Subcode
			arg1 = (lowB >> 5) + ((highB & 0x3) << 3);
			arg2 = (lowB & 0x1F);
			arg3 = (highB2 >> 2);
			if(arg3 > 16) {
				fprintf(outfile, "\n%08x\tError", pc);
			} else {
				fprintf(outfile, "\n%08x\t%s  $%d, $%d", pc, fpsuboptable[arg3], arg1, arg2);
			}
				pc += 4;   // 32 bit instruction
			break;
		case AM_UDEF:  // Invalid opcode.
		default:	   // Invalid opcode.
			fprintf(outfile, "\n%08x\t\t%02x %02x ;Invalid Opcode", pc, lowB, highB);
			pc += 2;					       
		}
	}

	fclose(infile);							 // Close files
	fclose(outfile);

	printf("\nDisassembly complete.\n");
} // End Main


