///////////////////////////////////////////////////////////////
// File:  v810_ds2.h
// 
// Author:  Bob VanderClay (vandercl@umich.edu)
//          David Tucker (dbt@dana.ucc.nau.edu)
// Description:  Defines used in v810_dis.cpp(not)
//
//
// History:
// --------
// 8 Apr 1998 - Created
// 9 Apr 1998 - Version 1.0 completed
// Many mods byb David Tucker....
//

#ifndef V810_DS2_H_
#define V810_DS2_H_  

#define WORD  unsigned long     //Full 32bit
#define HWORD unsigned short    //16bit
#define BYTE  unsigned char	  //8bit

extern WORD sign_26(WORD num);
extern WORD sign_16(HWORD num);
extern WORD sign_9(HWORD num);
extern WORD sign_8(BYTE num);
extern WORD sign_5(BYTE num);


///////////////////////////////////////////////////////////////////
// Define Modes
#define AM_I    1
#define AM_II   2
#define AM_III  3
#define AM_IV   4
#define AM_V    5
#define AM_VIa  61    // Mode6 form1
#define AM_VIb  62    // Mode6 form2
#define AM_VII  7
#define AM_VIII 8
#define AM_IX   9
#define AM_BSTR 0x0E  // Bit String Instructions
#define AM_FPP  0x0F  // Floating Point Instructions
#define AM_UDEF 0xFF  // Unknown/Undefined Instructions




///////////////////////////////////////////////////////////////////
// Table of Instructions and Address Modes

typedef struct op {
   int addr_mode;               // Addressing mode
   char * opname;               // optcode name (string)
} operation;

static operation optable[80] = {
  { AM_I,       "mov  "},           // 0x00
  { AM_I,       "add  "},           // 0x01
  { AM_I,       "sub  "},           // 0x02
  { AM_I,       "cmp  " },           // 0x03
  { AM_I,       "shl  " },           // 0x04
  { AM_I,       "shr  " },           // 0x05
  { AM_I,       "jmp  " },           // 0x06
  { AM_I,       "sar  " },           // 0x07
  { AM_I,       "mul  " },           // 0x08
  { AM_I,       "div  " },           // 0x09
  { AM_I,       "mulu " },           // 0x0A
  { AM_I,       "divu " },           // 0x0B
  { AM_I,       "or   " },           // 0x0C
  { AM_I,       "and  " },           // 0x0D
  { AM_I,       "xor  " },           // 0x0E
  { AM_I,       "not  " },           // 0x0F

  { AM_II,      "mov  " },           // 0x10  // Imediate
  { AM_II,      "add  " },           // 0x11
  { AM_II,      "setf " },           // 0x12
  { AM_II,      "cmp  " },           // 0x13
  { AM_II,      "shl  " },           // 0x14
  { AM_II,      "shr  " },           // 0x15
  { AM_II,      "EI   " },           // 0x16   
  { AM_II,      "sar  " },           // 0x17
  { AM_II,      "trap " },           // 0x18

  { AM_IX,      "reti " },           // 0x19  //BRKRETI
  { AM_IX,      "halt " },           // 0x1A  //STBY
  
  {AM_UDEF,     "???  " },           // 0x1B  // Unknown
  { AM_II,      "ldsr " },           // 0x1C
  { AM_II,      "stsr " },           // 0x1D
  { AM_II,      "DI   " },           // 0x1E   
  {AM_BSTR,     "BSTR " },           // 0x1F  // Special Bit String Instructions

  {AM_UDEF,     "???  " },           // 0x20  // Unknown   // This is a fudg on our part
  {AM_UDEF,     "???  " },           // 0x21  // Unknown   // We have 6 and 7 bit instructions 
  {AM_UDEF,     "???  " },           // 0x22  // Unknown   // this is filld in by the Conditional Branch Instructions
  {AM_UDEF,     "???  " },           // 0x23  // Unknown
  {AM_UDEF,     "???  " },           // 0x24  // Unknown
  {AM_UDEF,     "???  " },           // 0x25  // Unknown
  {AM_UDEF,     "???  " },           // 0x26  // Unknown
  {AM_UDEF,     "???  " },           // 0x27  // Unknown

  { AM_V,       "movea" },           // 0x28
  { AM_V,       "addi " },           // 0x29
  { AM_IV,      "jr   " },           // 0x2A
  { AM_IV,      "jal  " },           // 0x2B
  { AM_V,       "ori  " },           // 0x2C
  { AM_V,       "andi " },           // 0x2D
  { AM_V,       "xori " },           // 0x2E
  { AM_V,       "movhi" },           // 0x2F
  { AM_VIa,     "ld.b " },           // 0x30
  { AM_VIa,     "ld.h " },           // 0x31
  {AM_UDEF,     "muli " },           // 0x32  
  { AM_VIa,     "ld.w " },           // 0x33
  { AM_VIb,     "st.b " },           // 0x34
  { AM_VIb,     "st.h " },           // 0x35
  {AM_UDEF,     "maci " },           // 0x36  // Unknown
  { AM_VIb,     "st.w " },           // 0x37
  { AM_VIa,     "in.b " },           // 0x38
  { AM_VIa,     "in.h " },           // 0x39
  { AM_VIa,     "caxi " },           // 0x3A
  { AM_VIa,     "in.w " },           // 0x3B
  { AM_VIb,     "out.b" },           // 0x3C
  { AM_VIb,     "out.h" },           // 0x3D
  { AM_FPP,     "FPP  " },           // 0x3E  //Floating Point Instruction, Special Case
  { AM_VIb,     "out.w" },           // 0x3F

  { AM_III,     "bv   " },           // 0x40
  { AM_III,     "bl   " },           // 0x41  //BC  0x41
  { AM_III,     "be   " },           // 0x42  //BZ  0x42
  { AM_III,     "bnh  " },           // 0x43
  { AM_III,     "bn   " },           // 0x44
  { AM_III,     "br   " },           // 0x45
  { AM_III,     "blt  " },           // 0x46
  { AM_III,     "ble  " },           // 0x47
  { AM_III,     "bnv  " },           // 0x48
  { AM_III,     "bnl  " },           // 0x49 //BNC 0x49
  { AM_III,     "bne  " },           // 0x4A //BNZ 0x4A
  { AM_III,     "bh   " },           // 0x4B
  { AM_III,     "bp   " },           // 0x4C
  { AM_III,     "nop  " },           // 0x4D
  { AM_III,     "bge  " },           // 0x4E
  { AM_III,     "bgt  " },           // 0x4F
};
// All instructions greater than 0x50 are undefined (this should not be posible of cource)

//  Bit String Subopcodes
static char * bssuboptable[16] = {
  "SCH0BSU",                    // 0x00
  "SCH0BSD",                    // 0x01
  "SCH1BSU",                    // 0x02
  "SCH1BSD",                    // 0x03
  "Error  ",                    // 0x04
  "Error  ",                    // 0x05
  "Error  ",                    // 0x06
  "Error  ",                    // 0x07
  "ORBSU  ",                    // 0x08
  "ANDBSU ",                    // 0x09
  "XORBSU ",                    // 0x0A
  "MOVBSU ",                    // 0x0B
  "ORNBSU ",                    // 0x0C
  "ANDNBSU",                    // 0x0D
  "XORNBSU",                    // 0x0E
  "NOTBSU ",                    // 0x0F
};

//  Floating Point Subopcodes
static char * fpsuboptable[16] = {
  "cmpf.s ",                    // 0x00
  "Error  ",                    // 0x01
  "cvt.ws ",                    // 0x02
  "cvt.sw ",                    // 0x03
  "addf.s ",                    // 0x04
  "subf.s ",                    // 0x05
  "mulf.s ",                    // 0x06
  "divf.s ",                    // 0x07
  "Error  ",                    // 0x08
  "Error  ",                    // 0x09
  "Error  ",                    // 0x0A
  "trnc.sw",                    // 0x0B
  "Error  ",                    // 0x0C
  "Error  ",                    // 0x0D
  "Error  ",                    // 0x0E
  "Error  ",                    // 0x0F
};
///////////////////////////////////////////////////////////////////
// Opcodes for V810 Instruction set
#define         JMP                             0x06
/* //Dont need thes...
#define         MOV                             0x00
#define         ADD                             0x01
#define         SUB                             0x02
#define         CMP                             0x03
#define         SHL                             0x04
#define         SHR                             0x05
#define         JMP                             0x06
#define         SAR                             0x07
#define         MUL                             0x08
#define         DIV                             0x09
#define         MULU                            0x0A
#define         DIVU                            0x0B
#define         OR                              0x0C
#define         AND                             0x0D
#define         XOR                             0x0E
#define         NOT                             0x0F
#define         MOV_I                           0x10
#define         ADD_I                           0x11
#define         SETF                            0x12
#define         CMP_I                           0x13
#define         SHL_I                           0x14
#define         SHR_I                           0x15
#define			EI                              0x16
#define         SAR_I                           0x17
#define         TRAP                            0x18
#define         RETI                            0x19
#define         HALT                            0x1A
                                              //0x1B
#define         LDSR                            0x1C
#define         STSR                            0x1D
#define			DI                              0x1E
#define         BSTR                            0x1F  //Special Bit String Inst
                                              //0x20 - 0x27  // Lost to Branch Instructions
#define         MOVEA                           0x28
#define         ADDI                            0x29
#define         JR                              0x2A
#define         JAL                             0x2B
#define         ORI                             0x2C
#define         ANDI                            0x2D
#define         XORI                            0x2E
#define         MOVHI                           0x2F
#define         LD_B                            0x30
#define         LD_H                            0x31
//#define		MULI                            0x32
#define         LD_W                            0x33
#define         ST_B                            0x34
#define         ST_H                            0x35
//#define		MACI                            0x36
#define         ST_W                            0x37
#define         IN_B                            0x38
#define         IN_H                            0x39
#define         CAXI                            0x3A
#define         IN_W                            0x3B
#define         OUT_B                           0x3C
#define         OUT_H                           0x3D
#define         FPP                             0x3E  //Special Float Inst
#define         OUT_W                           0x3F


//      Branch Instructions ( Extended opcode only for Branch command)
//  Common instrcutions commented out

#define         BV                              0x40
#define         BL                              0x41
#define         BE                              0x42
#define         BNH                             0x43
#define         BN                              0x44
#define         BR                              0x45
#define         BLT                             0x46
#define         BLE                             0x47
#define         BNV                             0x48
#define         BNL                             0x49
#define         BNE                             0x4A
#define         BH                              0x4B
#define         BP                              0x4C
#define         NOP                             0x4D
#define         BGE                             0x4E
#define         BGT                             0x4F

//#define       BC                              0x41
//#define       BZ                              0x42
//#define       BNC                             0x49
//#define       BNZ                             0x4A

//  Bit String Subopcodes
#define         SCH0BSU                         0x00
#define         SCH0BSD                         0x01
#define         SCH1BSU                         0x02
#define         SCH1BSD                         0x03

#define         ORBSU                           0x08
#define         ANDBSU                          0x09
#define         XORBSU                          0x0A
#define         MOVBSU                          0x0B
#define         ORNBSU                          0x0C
#define         ANDNBSU                         0x0D
#define         XORNBSU                         0x0E
#define         NOTBSU                          0x0F


//  Floating Point Subopcodes
#define         CMPF_S                          0x00

#define         CVT_WS                          0x02
#define         CVT_SW                          0x03
#define         ADDF_S                          0x04
#define         SUBF_S                          0x05
#define         MULF_S                          0x06
#define         DIVF_S                          0x07

#define         TRNC_SW                         0x0B
*/ //the end...

#endif //DEFINE_H

